/* ************************************************************************** 
 *             The Pebbles Personal Digital Assistant Project               *
 * **************************************************************************
 * This code was written as part of the Pebbles project at                  *
 * Carnegie Mellon University.  See the following for more information:     *
 *                http://www.cs.cmu.edu/~pebbles                            *
 * This code is being made available for free general non-commercial use.   *
 * For other conditions, see http://www.cs.cmu.edu/~pebbles/conditions.html *
 * Please contact us for more information: pebbles@cs.cmu.edu               *  
 *         Copyright (C) 1998-2000 - Carnegie Mellon University                  *
 * **************************************************************************/

/*
	A sample program designed to connect with Pebbles via the PebblesSocket
	interface.  This program receives a Ping message from the PDA and returns
	the data from the Ping back to the PDA.

	Designed and implemented by Ben Bostwick
*/

#include "stdafx.h"
#include "resource.h"
#include <stdio.h>

#include "Pebbles.h"
#include "PebblesSocket.h"
#include "PingMessages.h"

#define MAX_LOADSTRING 100

#define HOSTNAME "127.0.0.1" 
#define PORTNUM 4242
#define TIMER_ID 100

#define PLUGIN_NAME "Ping"

// Global Variables:
HINSTANCE hInst;								// current instance
TCHAR szTitle[MAX_LOADSTRING];								// The title bar text
TCHAR szWindowClass[MAX_LOADSTRING];								// The title bar text

HWND hMainWnd = NULL;

HWND hTextField = NULL;

// Foward declarations of functions included in this code module:
ATOM				MyRegisterClass(HINSTANCE hInstance);
BOOL				InitInstance(HINSTANCE, int);
BOOL				InitSockets();
LRESULT CALLBACK	WndProc(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK	About(HWND, UINT, WPARAM, LPARAM);



/* add text to edit field */
void AddText(char *text) {
	int len = SendMessage(hTextField, WM_GETTEXTLENGTH, 0, 0);
	SendMessage(hTextField, EM_SETSEL, len, len); 
	SendMessage(hTextField, EM_REPLACESEL, FALSE, (LPARAM) text);
	SendMessage(hTextField, EM_REPLACESEL, FALSE, (LPARAM) "\r\n");
}


/********************************************************
 *
 *  BlockingRead
 *
 *  Reads data from the socket, blocking until all 
 *  requested bytes are received (or an error occurs).
 *  Returns number of bytes read.
 *
 *********************************************************/

int BlockingRead (char *data, int len) {
    int n = 0;

    while (n < len) {
        int nRead = len - n;
		if (!PebblesSocketReceive (data, &nRead)) {
			return n;
		}

        data += nRead;
        n += nRead;
    }
	
	return n;
}


/********************************************************
 *
 *  ReceiveCheck
 *
 *  This function parses the data received by a call
 *  to PebblesSocketReceive().  When this program receives
 *  a CMD_PING message, it returns a CMD_PONG message with
 *  the data that was sent with the CMD_PING.
 *
 *********************************************************/

void ReceiveCheck(char *header) {	
	char szDisplay[200];
	char *retMsg;

	char cmd = PEBBLES_CMD(header);
	int msgLen = PEBBLES_LEN(header);
	char id = PEBBLES_GET_USERID(header);

	sprintf(szDisplay, "Received command %d, data length %d", cmd, msgLen);
	AddText(szDisplay);

	char *data = new char[msgLen];
	int nRead = BlockingRead (data, msgLen);
	if (nRead != msgLen) {
		sprintf(szDisplay, "Read only %d bytes out of %d", nRead, msgLen);
		AddText(szDisplay);
		delete[] data;
		return;
	}

	switch(cmd) {
	case CMD_SOCKET_NEW_USER:
		// this is sent by PebblesSocket so an application
		// can keep track of the users that are connected
		sprintf(szDisplay, "...new user %d", id);
		AddText(szDisplay);
	break;
	case CMD_PING:
		AddText ("...ping");
		MessageBeep(0);

		// send the pong back to the PDA
		retMsg = new char[msgLen+PEBBLES_SOCKET_HEADER_SIZE];

		// set the command to be sent
		PEBBLES_SET_CMD(retMsg, CMD_PONG);

		// set the length of the data
		PEBBLES_SET_LEN(retMsg, msgLen);

		//set the userid of the PDA to send the message to.
		PEBBLES_SET_USERID(retMsg, id);

		// copy the message that was sent into the return message
		memcpy(PEBBLES_SOCKET_MESSAGE(retMsg), data, msgLen);
		PebblesSocketSend(retMsg, msgLen+PEBBLES_SOCKET_HEADER_SIZE);
	break;

	case CMD_SOCKET_DONE_USER:
		// notification from PebblesSocket that a user has left Pebbles
		sprintf(szDisplay, "...done user %d", id);
		AddText(szDisplay);
	break;
	default:
		AddText ("...unknown message");
		break;
	}

	delete[] data;
}


//
//  FUNCTION: WndProc(HWND, unsigned, WORD, LONG)
//
//  PURPOSE:  Processes messages for the main window.
//
//  WM_COMMAND	- process the application menu
//  WM_DESTROY	- post a quit message and return
//
//
LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	int wmId, wmEvent;
	int *recLen = new int;

	switch (message) 
	{
		case WM_CREATE:
			hTextField = CreateWindow("EDIT", NULL, WS_CHILD | WS_VISIBLE | WS_VSCROLL | WS_BORDER |
							ES_MULTILINE | ES_LEFT | ES_AUTOVSCROLL | ES_READONLY,
							0, 0, 0, 0, hWnd, NULL, hInst, NULL);
			AddText("Socket Plugin Test Program");
			AddText(" ");
			break;
		case WM_COMMAND:
			wmId    = LOWORD(wParam); 
			wmEvent = HIWORD(wParam); 
			// Parse the menu selections:
			switch (wmId)
			{
				case IDM_ABOUT:
				   DialogBox(hInst, (LPCTSTR)IDD_ABOUTBOX, hWnd, (DLGPROC)About);
				   break;
				case IDM_EXIT:
				   DestroyWindow(hWnd);
				   break;


				case IDM_CONNECT:
					// Connect to the server
					if(PebblesSocketConnect(PLUGIN_NAME, HOSTNAME, PORTNUM))
						AddText("Connected to Pebbles!");
					else
						AddText("Connection to Pebbles Failed!");
					break;

				case IDM_SENDSTRING:
					// Send a test string to the server
					PebblesSocketSend("blah", 4);
					break;


				default:
				   return DefWindowProc(hWnd, message, wParam, lParam);
			}
			break;
		case WM_SIZE:             
			// Make the edit control the size of the window's 
            // client area.             
			MoveWindow(hTextField, 
                0, 0,           // starting x- and y-coordinates 
                LOWORD(lParam), // width of client area 
                HIWORD(lParam), // height of client area 
                TRUE);          // repaint window             
			break; 

		case WM_DESTROY:
			PebblesSocketClose();
			PostQuitMessage(0);
			break;


		case WM_TIMER:
			// check to see if any data is available 
			if(PebblesSocketReceiveCheck()) {
				// data is available.. get the header
				char header[PEBBLES_SOCKET_HEADER_SIZE];

				if (BlockingRead (header, sizeof(header)) == sizeof(header))
					ReceiveCheck (header);
			}
			break;
		default:
			return DefWindowProc(hWnd, message, wParam, lParam);
   }
   return 0;
}


/******************************************
 *
 * WinMain -- call PebblesSocketInit() and 
 * start the timer here.
 *
 ******************************************/

int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow)
{
 

	MSG msg;
	HACCEL hAccelTable;

	// Initialize global strings
	LoadString(hInstance, IDS_APP_TITLE, szTitle, MAX_LOADSTRING);
	LoadString(hInstance, IDC_RECEIVESOCKET, szWindowClass, MAX_LOADSTRING);
	MyRegisterClass(hInstance);

	// Perform application initialization:
	if (!InitInstance (hInstance, nCmdShow)) 
	{
		return FALSE;
	}

	hAccelTable = LoadAccelerators(hInstance, (LPCTSTR)IDC_RECEIVESOCKET);

	// initialize sockets
	if(!PebblesSocketInit()) {
		return FALSE;
	}

	// set the ReceiveCheck timer
	SetTimer(hMainWnd, TIMER_ID, 500, NULL);

	// Main message loop:
	while (GetMessage(&msg, NULL, 0, 0)) 
	{
		if (!TranslateAccelerator(msg.hwnd, hAccelTable, &msg)) 
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}

	return msg.wParam;
}


/* Standard Windows garbage... */


// Mesage handler for about box.
LRESULT CALLBACK About(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
		case WM_INITDIALOG:
				return TRUE;

		case WM_COMMAND:
			if (LOWORD(wParam) == IDOK || LOWORD(wParam) == IDCANCEL) 
			{
				EndDialog(hDlg, LOWORD(wParam));
				return TRUE;
			}
			break;
	}
    return FALSE;
}

//
//  FUNCTION: MyRegisterClass()
//
//  PURPOSE: Registers the window class.
//
//  COMMENTS:
//
//    This function and its usage is only necessary if you want this code
//    to be compatible with Win32 systems prior to the 'RegisterClassEx'
//    function that was added to Windows 95. It is important to call this function
//    so that the application will get 'well formed' small icons associated
//    with it.
//
ATOM MyRegisterClass(HINSTANCE hInstance)
{
	WNDCLASSEX wcex;

	wcex.cbSize = sizeof(WNDCLASSEX); 

	wcex.style			= CS_HREDRAW | CS_VREDRAW;
	wcex.lpfnWndProc	= (WNDPROC)WndProc;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= 0;
	wcex.hInstance		= hInstance;
	wcex.hIcon			= LoadIcon(hInstance, (LPCTSTR)IDI_RECEIVESOCKET);
	wcex.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground	= (HBRUSH)(COLOR_WINDOW+1);
	wcex.lpszMenuName	= (LPCSTR)IDC_RECEIVESOCKET;
	wcex.lpszClassName	= szWindowClass;
	wcex.hIconSm		= LoadIcon(wcex.hInstance, (LPCTSTR)IDI_SMALL);

	return RegisterClassEx(&wcex);
}

//
//   FUNCTION: InitInstance(HANDLE, int)
//
//   PURPOSE: Saves instance handle and creates main window
//
//   COMMENTS:
//
//        In this function, we save the instance handle in a global variable and
//        create and display the main program window.
//
BOOL InitInstance(HINSTANCE hInstance, int nCmdShow)
{
 

   hInst = hInstance; // Store instance handle in our global variable

   hMainWnd = CreateWindow(szWindowClass, szTitle, WS_OVERLAPPEDWINDOW,
      CW_USEDEFAULT, 0, CW_USEDEFAULT, 0, NULL, NULL, hInstance, NULL);

   if (!hMainWnd)
   {
      return FALSE;
   }

   ShowWindow(hMainWnd, nCmdShow);
   UpdateWindow(hMainWnd);

   return TRUE;
}