//
// Pebbles library for WinCE Devices.
//
// This file defines a set of functions that implement the app-side
// of the Pebbles protocol for a WinCE Device.
//

#include <windows.h>
#include <commctrl.h>
#include <winsock.h>
#include "Pebbles.h"
#include "PebblesCE.h"

#ifndef SD_BOTH
#define SD_BOTH 0x02 // this should be defined in winsock.h, but it isn't.
#endif

#define HTOS(handle) (*((SOCKET *) handle)) // converts a handle (void *) to a SOCKET


// Interval for repeating the CHANGE_PLUGIN message, in milliseconds
#define CHANGE_PLUGIN_REPEAT_INTERVAL_MSECS     2000

#define NETWORK_RECONNECT_INTERVAL_MSECS   5000

// beep when connection is made
#define CONNECTED_SOUND

//#define PEBBLES_DEBUG

#ifdef PEBBLES_DEBUG
HANDLE debugFile = NULL;

void DebugWriteFile(char *message, int messagelen) {
	DWORD b;
	
	WriteFile(debugFile, message, messagelen, &b, NULL);
	SetEndOfFile(debugFile);
}
#endif

void ChangeCommRegistry(BOOL change) {
	DWORD err;
	HKEY hNew = NULL;
	unsigned char data = (unsigned char) change;

    err = RegOpenKeyEx(HKEY_CURRENT_USER, 
			TEXT("ControlPanel\\Comm"), 
			0,
			(KEY_READ | KEY_WRITE),
			&hNew);
		if(err == ERROR_SUCCESS) {
			 RegSetValueEx(hNew, TEXT("AutoCnct"), 0, REG_BINARY, &data, 1);
			 RegCloseKey(hNew);
        } 
		//else MessageBox(NULL, TEXT("Can't Open Registry!"), TEXT("Error"), MB_OK);

}

static char *defaultUsername = NULL;
void PebblesSetUserName (const char *name) {
	if (defaultUsername)
		delete[] defaultUsername;
	defaultUsername = new char[strlen(name)+1];
	strcpy (defaultUsername, name);
}


void GetOwnerName(char *nameBuf, DWORD *nameLen) {
	if (defaultUsername) {
		strcpy (nameBuf, defaultUsername);
		*nameLen = strlen (nameBuf);
		return;
	}

	DWORD err;
	HKEY hNew = NULL;
	DWORD dType;
	unsigned char tmpBuf[70];
	unsigned int i;

	*nameLen = 70;

	// this is the name that the CE device uses to connect to
	// the PC with... not the owner name
	err = RegOpenKeyEx(HKEY_LOCAL_MACHINE, 
			TEXT("Ident"), 
			0,
			KEY_READ,
			&hNew);
		if(err == ERROR_SUCCESS) {
			err = RegQueryValueEx(hNew, TEXT("Name"), 0, &dType, tmpBuf, nameLen);
			if(err == ERROR_SUCCESS) {
				*nameLen = *nameLen/2;
				for(i=0;i<*nameLen;i++) nameBuf[i]=tmpBuf[2*i];
			 } else {
				*nameLen = 0;
			 }
			 RegCloseKey(hNew);
        } else {
			//MessageBox(NULL, TEXT("Can't Open Registry!"), TEXT("Error"), MB_OK);
			*nameLen = 0;
		}

}


HANDLE OpenSocket(PebblesPtr pbl) {
	SOCKET *pblSocket;
	int index = 0;                     // An integer 
	TCHAR szError[100];                 // Error message string 
	SOCKADDR_IN saddrPebbles;        // Server socket address 
	PHOSTENT phostent = NULL;           // Points to the HOSTENT structure 
									  // of the server 

	pblSocket = new SOCKET;
	*pblSocket = INVALID_SOCKET;

	// Create a TCP/IP socket bound to the server.
	if ((*pblSocket = socket (AF_INET, SOCK_STREAM, NULL)) == INVALID_SOCKET)  {
		wsprintf (szError, TEXT("Allocating socket failed. Error: %d"), WSAGetLastError ());
		MessageBox (NULL, szError, TEXT("Error"), MB_OK);
		WSACleanup();
		return NULL;
	}

	// set the socket into non-blocking mode;
	u_long nonBlocking = 1;
	if(ioctlsocket (*pblSocket, FIONBIO, &nonBlocking) == SOCKET_ERROR) {
		wsprintf (szError, TEXT("IO Control Set Failed. Error: %d"), WSAGetLastError ()); 
		MessageBox (NULL, szError, TEXT("Error"), MB_OK); 
		closesocket(*pblSocket);
		WSACleanup();
		return NULL;
	}

	// set the address
	saddrPebbles.sin_port = htons(PEBBLES_APP_PORTNUM); // port number to connect to
	saddrPebbles.sin_family = AF_INET;

	UINT addr;
	int err, ret;
/*
	addr = inet_addr(networkAddr);
	phostent = gethostbyaddr((char *)&addr,4,AF_INET);
	if(phostent == NULL) {
		err = WSAGetLastError();
		wsprintf (szError, TEXT("Resolving the address failed. Error: %d"), err); 
		MessageBox (NULL, szError, TEXT("Error"), MB_OK); 
	}
	memcpy ((char FAR *)&(saddrPebbles.sin_addr), phostent->h_addr, phostent->h_length);  // Convert to network ordering.
*/
	// First try to interpret ipAddr as an IP address
	addr = inet_addr (pbl->ipAddr);
	saddrPebbles.sin_addr.S_un.S_addr = addr;

	if (addr == INADDR_NONE) {
		// if it isn't an IP address, try it as a hostname
		if ((phostent = gethostbyname (pbl->ipAddr)) == NULL)   {
			wsprintf (szError, TEXT("Unknown host '%s'. Error: %d"), pbl->ipAddr, WSAGetLastError ());
			MessageBox (NULL, szError, TEXT("Error"), MB_OK);
			closesocket (*pblSocket);
			WSACleanup();
			return NULL;
		}

		memcpy (&saddrPebbles.sin_addr.s_addr, phostent->h_addr_list[0],
                     phostent->h_length);
	}

	if(connect(*pblSocket, (LPSOCKADDR) &saddrPebbles, sizeof(SOCKADDR_IN)) == SOCKET_ERROR) {
		err = WSAGetLastError();
		if(err == WSAEWOULDBLOCK) {
			// block until the connection is ready

			FD_SET fd;
			FD_ZERO(&fd);
			FD_SET(*pblSocket, &fd);        // add socket to the socket set
	
			TIMEVAL tm = {CHANGE_PLUGIN_REPEAT_INTERVAL_MSECS/1000, 0};

			// wait for data
			if ((ret = select(0, NULL, &fd, NULL, &tm)) != SOCKET_ERROR) {
				// connection is available
				if(ret > 0) {
					// socket is connected
					return pblSocket;
				}
			} 
			else { 
				// call to select failed
				err = WSAGetLastError();
			}
		} 
		// can't connect to the server
		wsprintf (szError, TEXT("Connecting to the server failed. Error: %d"), err); 
		//MessageBox (NULL, szError, TEXT("Error"), MB_OK); 
		closesocket(*pblSocket);
		WSACleanup();
		return NULL;
	}

/*
	// Fill out the server socket address data. 
	destination_sin.sin_family = AF_INET;
	// Retrieve the host data corresponding to the host name. 
	if ((phostent = gethostbyname (HOSTNAME)) == NULL)   {
		wsprintf (szError, TEXT("Unable to get the host name. Error: %d"), WSAGetLastError ());
		MessageBox (NULL, szError, TEXT("Error"), MB_OK);
		closesocket (*pblSocket);
		WSACleanup();
		return NULL;
	}

	// Assign the socket IP address. 
	memcpy ((char FAR *)&(destination_sin.sin_addr), phostent->h_addr, phostent->h_length);  // Convert to network ordering.
	destination_sin.sin_port = htons (PORTNUM);
  
	// Establish a connection to the server socket. 
	if (connect (*pblSocket, (PSOCKADDR) &destination_sin, sizeof (destination_sin)) == SOCKET_ERROR)   {
		wsprintf (szError, TEXT("Connecting to the server failed. Error: %d"), WSAGetLastError ()); 
		MessageBox (NULL, szError, TEXT("Error"), MB_OK); 
		closesocket (*pblSocket);
		WSACleanup();
		return NULL;
	}
*/	
	return pblSocket;
}

HANDLE OpenPort (PebblesConnectionType connType)
{
	HANDLE hPort;

	

#ifdef SH3
	TCHAR *szComPort = (connType == INFRARED) ? TEXT("COM4:") : TEXT("COM1:");
#else
	TCHAR *szComPort = TEXT("COM1:");
#endif

    hPort = CreateFile(szComPort,
					   (GENERIC_READ | GENERIC_WRITE),
					   0, NULL, OPEN_EXISTING,
					   0,
					   NULL);

    if (hPort == INVALID_HANDLE_VALUE )
		return NULL;

	// Get the port info.
	DCB		PebblesDCB;
	PebblesDCB.DCBlength = sizeof(DCB);
    if(!GetCommState(hPort, &PebblesDCB))
		return NULL;

	PebblesDCB.BaudRate = PEBBLES_BAUDRATE;
	PebblesDCB.fBinary = TRUE;
	PebblesDCB.fParity = TRUE;
	PebblesDCB.fOutxCtsFlow    = FALSE;  // ignore possible hangups
	PebblesDCB.fOutxDsrFlow    = FALSE;    // don't wait on the DSR line
	PebblesDCB.fDtrControl     = DTR_CONTROL_ENABLE;
	PebblesDCB.fDsrSensitivity = FALSE;
	PebblesDCB.fTXContinueOnXoff = TRUE;
	PebblesDCB.fOutX           = FALSE; // no XON/XOFF control
	PebblesDCB.fInX            = FALSE;
	PebblesDCB.fErrorChar      = FALSE;
	PebblesDCB.fNull           = FALSE;
	PebblesDCB.fRtsControl     = RTS_CONTROL_HANDSHAKE;
	PebblesDCB.fAbortOnError   = FALSE;
	PebblesDCB.ByteSize        = 8;
	PebblesDCB.Parity          = NOPARITY;
	PebblesDCB.StopBits        = ONESTOPBIT;

	// Set the port info
	if(!SetCommState(hPort, &PebblesDCB))
		return NULL;

#ifdef SETIR
	if (connType == INFRARED) 
            EscapeCommFunction(hPort, SETIR);
#endif

	return hPort;
}

DWORD WINAPI WatchForPowerOn (PebblesPtr pbl)
{
#ifdef EV_POWER
	DWORD mask;

	while (pbl->hPort != NULL) {
		// ask only for EV_POWER events
		SetCommMask (pbl->hPort, EV_POWER);

		if (WaitCommEvent (pbl->hPort, &mask, 0) &&
			(mask & EV_POWER) == EV_POWER) {
				// EV_POWER occured; i.e., the device just powered up.
				// Reopen the serial port.
				CloseHandle(pbl->hPort);
				pbl->hPort = OpenPort (pbl->connType);
		}
	}
#endif
	return 0;
}


/***********************************************************************
 *
 * FUNCTION:    PebblesOpen
 *
 * DESCRIPTION: Open a connection to the desktop.
 *
 * PARAMETERS:  pbl -- connection state (caller must allocate)
 *              plugin -- name of plugin to open on desktop.
 *                        PebblesOpen saves a pointer to this
 *                        string, so it must live as long as the
 *                        connection.  Can be NULL, in which case
 *                        PebblesOpen doesn't attempt to open a plugin.
 *              connType -- type of connection, either SERIAL or INFRARED.
 *              
 * RETURNED:    true if connection opened succesfully;
 *				false if error
 *              
 *        Note that a successful return from this function means only
 *        that the device connection is open.  It does not mean that
 *        the PDA is connected to a PC with a running PebblesPC
 *        process with the desired plugin.  PebblesHandshake() or
 *        PebblesReceiveHeader() must be called periodically to watch
 *        for the desktop plugin to appear on the other end of the 
 *        connection.
 *
 ***********************************************************************/

BOOL PebblesOpen (PebblesPtr pbl, char *plugin, PebblesConnectionType connType, char *networkAddr, int flags)
{

#ifdef PEBBLES_DEBUG
	if(debugFile == NULL) {
		debugFile = CreateFile(TEXT("\\PebblesDebug.txt"),
					   (GENERIC_READ | GENERIC_WRITE),
					   0, NULL, CREATE_ALWAYS,
					   0,
					   NULL);
	}
	char szError[100];
	sprintf(szError, "%d: Pebbles Open!\n", GetTickCount());
	DebugWriteFile(szError, strlen(szError));
#endif

	// initialize PebblesPtr
	pbl->plugin = plugin;
	pbl->flags = flags;
	pbl->hPort = NULL;
	pbl->connected = FALSE;
	pbl->everConnected = FALSE;
    pbl->pluginConnectError = FALSE;
	pbl->connType = connType;
    pbl->connectionCallback = NULL;
	pbl->hWatchdogThread = NULL;
	pbl->ipAddr = networkAddr;

	if(connType == NETWORK) {
		WSADATA WSAData;
		
		// Initiate Windows Sockets.   
		if (WSAStartup (MAKEWORD(1,1), &WSAData) != 0)  {			
#ifdef PEBBLES_DEBUG
			sprintf (szError, "WSAStartup failed. Error: %d\n", WSAGetLastError ());
			DebugWriteFile(szError, strlen(szError));
#endif
			return NULL;
		}

		pbl->hPort = OpenSocket(pbl);
		if(pbl->hPort == NULL) {
			pbl->repeatDeadline = GetTickCount() + NETWORK_RECONNECT_INTERVAL_MSECS;
#ifdef PEBBLES_DEBUG			
			sprintf(szError, "Pebbles Open Socket failed!\n");
			DebugWriteFile(szError, strlen(szError));
#endif
			return FALSE;
		}
	}
	else {
		ChangeCommRegistry(FALSE);
		pbl->hPort = OpenPort(connType);
		if(pbl->hPort == NULL)
			return FALSE;

		pbl->hWatchdogThread = CreateThread (0, 0, (LPTHREAD_START_ROUTINE) WatchForPowerOn, pbl, 0, 0);

	}

	if (plugin != NULL)
		return (PebblesChangePlugin (pbl, plugin, flags));

	return TRUE;
}	


/***********************************************************************
 *
 * FUNCTION:    PebblesClose
 *
 * DESCRIPTION: Closes connection to the desktop.
 *
 * PARAMETERS:  pbl	-- connection state
 *
 * RETURNED:    TRUE if closed correctly, FALSE otherwise
 *
 ***********************************************************************/


void SerialClose(PebblesPtr pbl) {
	CloseHandle(pbl->hPort);
	// enable PC link
	ChangeCommRegistry(TRUE);
}

void SocketClose(PebblesPtr pbl) {

	// close port
	if(pbl->hPort) {
		shutdown (HTOS(pbl->hPort), SD_BOTH);  // Close the socket for both reading and writing 
		closesocket (HTOS(pbl->hPort));  
		WSACleanup ();					// windows sockets cleanup
	}

}

void PebblesClose (PebblesPtr pbl)
{
#ifdef PEBBLES_DEBUG	
	char szDebug[50];
	sprintf(szDebug, "%d: Pebbles Close!\n", GetTickCount());
	DebugWriteFile(szDebug, strlen(szDebug));
	//CloseHandle(debugFile);
#endif

	// tell Pebbles that we're going away (only if connected)
	if(pbl->connected) 
		PebblesChangePlugin (pbl, NULL);

	// close port
	if(pbl->connType == NETWORK) {
		SocketClose(pbl);
	}
	else {
		// kill watchdog thread
		CloseHandle (pbl->hWatchdogThread);
		SerialClose(pbl);
	}
	pbl->hPort = NULL;
	pbl->connected = false;
	
}


/***********************************************************************
 *
 * FUNCTION:    PebblesChangePlugin
 *
 * DESCRIPTION: Open a new plugin on the desktop (closing the
 *              current plugin, if any).
 *
 * PARAMETERS:  pbl -- connection state
 *              plugin -- name of plugin to open on desktop.
 *                        PebblesOpen saves a pointer to this
 *						  string, so it must live as long as the
 *                        connection.  Can be NULL, in which case
 *                        the current plugin is just closed.
 * RETURNED:    nothing
 *
 ***********************************************************************/

BOOL PebblesChangePlugin (PebblesPtr pbl, char *plugin, int flags)
{
	BOOL ret;
	unsigned plugLen = (plugin != NULL) ? strlen (plugin) : 0;

	char username[30];
	DWORD userNameLen;
	GetOwnerName(username, &userNameLen);

	// fool PebblesSend into thinking we're connected
	pbl->connected = TRUE;
	/*
	ret = PebblesSendHeader (pbl, CMD_PEBBLES_CHANGE_PLUGIN, plugLen+userNameLen+1);
	ret &= PebblesSendData(pbl, plugin, plugLen);
	ret &= PebblesSendData(pbl, "\0", 1);
	ret &= PebblesSendData(pbl, userName, userNameLen);
	*/
	char dataToSend[30];
	PEBBLES_SET_CMD (dataToSend, CMD_PEBBLES_CHANGE_PLUGIN);
	PEBBLES_SET_LEN (dataToSend, (plugLen+userNameLen+3));
	//PEBBLES_SET_LEN (dataToSend, (plugLen+userNameLen+1));
	memcpy((dataToSend+3), plugin, plugLen);
	dataToSend[3+plugLen] = '\0';
	memcpy((dataToSend+4+plugLen), username, userNameLen);
	dataToSend[3+plugLen+1+userNameLen] = '\0';

	if (pbl->connType != INFRARED)
		flags |= PEBBLES_PLUGIN_FLAG_FLOW_CONTROL;
	dataToSend[3+plugLen+1+userNameLen+1] = (char) flags;

	ret = PebblesSendData(pbl, dataToSend, (3+plugLen+1+userNameLen+2));
	//ret = PebblesSendData(pbl, dataToSend, (3+plugLen+1+userNameLen));
	

	pbl->connected = FALSE;
	pbl->plugin = plugin;

	pbl->repeatDeadline = GetTickCount() + CHANGE_PLUGIN_REPEAT_INTERVAL_MSECS;

	return (ret);
}

/***********************************************************************
 *
 * FUNCTION:    PebblesHandshake
 *
 * DESCRIPTION: Checks whether the desktop plugin is connected and
 *              available.
 *
 *				Apps should periodically call either this function
 *				or PebblesReceiveHeader to watch for handshaking 
 *              requests from the desktop.  Note that this function eats 
 *				all incoming messages, so an app which is expecting
 *				asynchronous messages should use PebblesReceiveHeader
 *              instead.
 *
 * PARAMETERS:  pbl -- connection state
 *				timeout -- interbyte timeout, in ticks (.01 sec).
 *						   0 means poll, -1 means wait forever.	
 *
 * RETURNED:    true if plugin connected;
 *              false if plugin not connected yet.
 *
 ***********************************************************************/


BOOL PebblesHandshake (PebblesPtr pbl)
{
	unsigned len;

	if (PebblesReceiveHeader(pbl, &len, 0))
		PebblesReceiveData (pbl, NULL, len, 50);

	return TRUE;
}

/***********************************************************************
 *
 * FUNCTION:    PebblesSend
 *
 * DESCRIPTION: Send a message to the desktop plugin.
 *
 * PARAMETERS:  pbl -- connection state
 *              cmd -- user-defined message code (at least CMD_PEBBLES_BASE)
 *              data -- pointer to data to send; may be NULL if len=0
 *              len -- length of data block, may be 0
 *
 * RETURNED:    true if message successfully sent to plugin;
 *              false if no plugin connected.
 *
 ***********************************************************************/

BOOL PebblesSend (PebblesPtr pbl, char cmd, void *data, unsigned len)
{
	/*return (PebblesSendHeader (pbl, cmd, len)
			&& PebblesSendData (pbl, data, len));
*/

	char *dataToSend = new char[PEBBLES_HEADER_SIZE+len];
	PEBBLES_SET_CMD (dataToSend, cmd);
	PEBBLES_SET_LEN (dataToSend, len);
	memcpy((dataToSend+3), data, len);

#ifdef PEBBLES_DEBUG
	char *szText = new char[PEBBLES_HEADER_SIZE+len+40];
	char tmp[10];
	sprintf(szText, "%d: Send: %d Len: %d Data: ", GetTickCount(), cmd, len);
	for(int i=0; i<((int) len); i++) {
		sprintf(tmp, "%02X ",((unsigned char *) data)[i]);
		strcat(szText, tmp);
	}
	sprintf(tmp, "\n");
	strcat(szText, tmp);
	DebugWriteFile(szText, strlen(szText));
	delete [] szText;
#endif

	BOOL ret = PebblesSendData(pbl, dataToSend, (3+len));
	delete [] dataToSend;
	return ret;

}

/***********************************************************************
 *
 * FUNCTION:    PebblesSendHeader
 *
 * DESCRIPTION: Send a message header to the plugin.
 *              Caller is responsible for finishing the message
 *              by sending <len> bytes of data using PebblesSendData().
 *
 * PARAMETERS:  pbl -- connection state
 *              cmd -- user-defined message code (at least CMD_PEBBLES_BASE)
 *              len -- length of data
 *
 * RETURNED:    true if message header successfully sent to plugin;
 *              false if no plugin connected.
 *
 ***********************************************************************/

BOOL SocketSendHeader (PebblesPtr pbl, void *hdr, unsigned len)
{

	int bytesSent = send (HTOS(pbl->hPort), (char *) hdr, sizeof(hdr), 0);
	if(bytesSent == SOCKET_ERROR) {
		pbl->connected = false;
		return false;
	}
	
	return (bytesSent == PEBBLES_HEADER_SIZE);
}

BOOL SerialSendHeader (PebblesPtr pbl, void *hdr, unsigned len)
{
	unsigned long Data;
	return (WriteFile (pbl->hPort, hdr, sizeof(hdr), &Data, NULL));
}

BOOL PebblesSendHeader (PebblesPtr pbl, char cmd, unsigned len)
{
	BOOL Err = false;
	char hdr[PEBBLES_HEADER_SIZE];

	if (!pbl->connected)
		return FALSE;		// no hope of delivering this message
				
	PEBBLES_SET_CMD (hdr, cmd);
	PEBBLES_SET_LEN (hdr, len);

	if(pbl->connType == NETWORK) {
		Err = SocketSendHeader(pbl, hdr, sizeof(hdr));
	}
	else {
		Err = SerialSendHeader(pbl, hdr, sizeof(hdr));
	}

	return Err;
}


/***********************************************************************
 *
 * FUNCTION:    PebblesSendData
 *
 * DESCRIPTION: Send message data to the plugin.  The message should
 *              have already been started by PebblesSendHeader.
 *              PebblesSendData can be called several times to send
 *              the complete data field of the message (as long as
 *              the total data sent equals the value passed to
 *              PebblesSendHeader).
 *
 * PARAMETERS:  pbl -- connection state
 *              data -- pointer to data to send
 *              len -- length of data block, may be 0
 *
 * RETURNED:    true if data successfully sent to plugin;
 *              false if no plugin connected.
 *
 ***********************************************************************/
BOOL SocketSendData (PebblesPtr pbl, void *data, unsigned len)
{	
	if (!pbl->hPort)
		return false;

	int bytesSent = send (HTOS(pbl->hPort), (char *) data, len, 0);
	if(bytesSent == SOCKET_ERROR) {
		DWORD error = WSAGetLastError();
		if(error != WSAEWOULDBLOCK) {
			pbl->connected = false;
#ifdef PEBBLES_DEBUG
			char szError[50];
			sprintf(szError, "Send Failed. Error: %d\n", error);
			DebugWriteFile(szError, strlen(szError));
#endif
		}
	}

	return ((unsigned) bytesSent == len);
}

BOOL SerialSendData (PebblesPtr pbl, void *data, unsigned len)
{
	unsigned long Data;
	return (WriteFile (pbl->hPort, data, len, &Data, NULL));
}

BOOL PebblesSendData (PebblesPtr pbl, void *data, unsigned len)
{
	if (!pbl->connected)
		return FALSE;	// no hope of delivering

	if(pbl->connType == NETWORK) {
		return SocketSendData(pbl, data, len);
	}
	else {
		return SerialSendData(pbl, data, len);
	}

	return TRUE;
}


/***********************************************************************
 *
 * FUNCTION:    PebblesSetConnectionCallback
 *
 * DESCRIPTION: Set a callback function to be called every time a
 *              new connection is established with PebblesPC (after
 *              CMD_PEBBLES_ACK_CHANGE_PLUGIN is received).
 *
 * PARAMETERS:  pbl -- connection state
 *              callback -- callback function.  May be NULL if no
 *                  callback is desired (the default).
 *
 *
 ***********************************************************************/

void PebblesSetConnectionCallback (PebblesPtr pbl, void (*callback) (PebblesPtr pbl)) 
{
    pbl->connectionCallback = callback;
}



// ReadWait waits until len characters are ready to be read.
static BOOL SocketReadWait (PebblesPtr pbl, unsigned long len, int timeout)
{
	fd_set      fdread;
	TIMEVAL		tmval;
	int         ret;

	if(pbl->hPort == NULL)
		return false;

	// convert timeout into seconds and microseconds
	tmval.tv_sec = timeout/1000;
	tmval.tv_usec = (timeout%1000)*10000;

	FD_ZERO(&fdread);          // clear the fd_set
	FD_SET(HTOS(pbl->hPort), &fdread);        // add socket to the socket set

	// wait for data
	if ((ret = select(0, &fdread, NULL, NULL, &tmval)) == SOCKET_ERROR) {
		pbl->connected = false;
		return FALSE;
	} 

	// if the socket is in the set, then data is waiting
	return (FD_ISSET(HTOS(pbl->hPort), &fdread));
}

// ReadWait waits until len characters are ready to be read.
static BOOL SerialReadWait (PebblesPtr pbl, unsigned long len, int timeout)
{
	COMSTAT stat;
	DWORD errors;

	if(pbl->hPort == NULL)
		return FALSE;

	if (!ClearCommError (pbl->hPort, &errors, &stat))
		return FALSE;

	if (stat.cbInQue >= len)
		return TRUE;

	if (timeout <= 0) 
		return FALSE;

	// Pebbles protocol specifies timeouts to
	// be .01 sec, while Wince wants .001 sec.
	Sleep (timeout * 10);

	if (!ClearCommError (pbl->hPort, &errors, &stat))
		return FALSE;

	return (stat.cbInQue >= len);

}


static BOOL ReadWait(PebblesPtr pbl, unsigned long len, int timeout) {
	if(pbl->connType == NETWORK)
		return SocketReadWait(pbl, len, timeout);
	else
		return SerialReadWait(pbl, len, timeout);
}

// ReadWithTimeout reads up to len characters into data buffer, subject to
// timeout (as defined above).  Returns number of chars read.
static unsigned long SocketReadWithTimeout (PebblesPtr pbl, char *data, 
									  unsigned long len, int timeout, void (*callback) (int percentComplete) = NULL)
{

	int iReturn;

	if(pbl->hPort == NULL)
		return 0;

	char *szReceive = new char[len];
	if (!szReceive) {
		TCHAR wbuf[80];
		wsprintf (wbuf, TEXT("Can't allocate %d bytes to receive message"), len);
		MessageBox (NULL, wbuf, TEXT("Error"), MB_ICONEXCLAMATION | MB_OK);
		return 0;        
	}

	int Err;

	// Receive data from the server socket with no timeout
	// a maximum of len bytes will be received
	iReturn = recv (HTOS(pbl->hPort), szReceive, len, 0);
	
	// Verify if data is received. 
	if (iReturn == SOCKET_ERROR)    {
		// there was an error.. most likely the socket closed
		Err = WSAGetLastError();

		if(Err == WSAEMSGSIZE) {
			// there is more data waiting in the queue
			if (data)
				memcpy(data, szReceive, len);
		}
		else {
			pbl->connected = false;
			delete[] szReceive;
			return 0;
		}
	}
	else    {
		float percentComplete = 0;
		// valid data was received, copy it into the return buffer	
		while(iReturn < (int) len) {
			if(SocketReadWait(pbl, (len-iReturn), timeout)) {
				iReturn += recv(HTOS(pbl->hPort), (szReceive+iReturn), (len-iReturn), 0);
				percentComplete = (float) (iReturn/len)*100;
				if(callback) callback((int) percentComplete);
			}
			else {
				break;
			}
		}
		if (data)
			memcpy(data, szReceive, iReturn);
	}	
	delete[] szReceive;
	return iReturn;
}

static unsigned long SerialReadWithTimeout (PebblesPtr pbl, char *data, 
									  unsigned long len, int timeout, void (*callback) (int percentComplete) = NULL)
{
		
    COMMTIMEOUTS cto;

    GetCommTimeouts(pbl->hPort, &cto);
    if(timeout == -1) {
	    cto.ReadIntervalTimeout = 0;
	    cto.ReadTotalTimeoutMultiplier = 0;
	    cto.ReadTotalTimeoutConstant = 0;
    } else if(timeout == 0) {
	    cto.ReadIntervalTimeout = MAXDWORD;
	    cto.ReadTotalTimeoutMultiplier = 0;
	    cto.ReadTotalTimeoutConstant = 0;
    } else {
	    // Pebbles protocol specifies timeouts to
	    // be .01 sec, while Wince wants .001 sec.
	    int msec = timeout * 10;

	    cto.ReadIntervalTimeout = MAXDWORD;
	    cto.ReadTotalTimeoutMultiplier = MAXDWORD;
	    cto.ReadTotalTimeoutConstant = msec;
    }
    SetCommTimeouts(pbl->hPort, &cto);

    // Gotta read character-by-character to get interval timeouts
    // working properly (the default interval timeout waits FOREVER
    // for the first byte -- duh)
    unsigned long n;
    char c;

	int percentComplete = 0;
    for (n = 0; n < len; ++n) {
        unsigned long was_read = 0;
        ReadFile (pbl->hPort, data != NULL ? data++ : &c, 1, &was_read, NULL);

		if(callback && (n%(len/10) == 0)) callback(percentComplete+=10); 

        if (!was_read)
            break;
    }

    return n;
}

static unsigned long ReadWithTimeout(PebblesPtr pbl, char *data, unsigned long len, int timeout, void (*callback) (int percentComplete) = NULL)
{
	if(pbl->connType == NETWORK) 
		return SocketReadWithTimeout(pbl, data, len, timeout, callback);
	else
		return SerialReadWithTimeout(pbl, data, len, timeout, callback);

}

/***********************************************************************
 *
 * FUNCTION:    PebblesReceiveHeader
 *
 * DESCRIPTION: Receives a message header from the desktop plugin.
 *              Depending on the timeout parameter, this function acts
 *              as either a nonblocking or a blocking read.
 *              Returns only user-defined messages (not system protocol
 *              messages).
 *
 *              If a message is returned, the caller is responsible
 *              for reading <*pLen> bytes from the serial port using
 *              PebblesReceiveData ().
 *
 * PARAMETERS:  pbl -- connection state
 *              pLen -- variable to receive length of incoming message's 
 *              		data block
 *				timeout -- interbyte timeout in ticks (.01 sec)
 * 				           0 means poll, -1 means wait forever
 *
 * RETURNED:    message code of received message;
 *              0 if no message is received.
 *
 ***********************************************************************/

char PebblesReceiveHeader (PebblesPtr pbl, unsigned int *pLen, int timeout)
{
	char hdr[PEBBLES_HEADER_SIZE];
	char cmd;
	unsigned long bytes_read = 0;
	
	if (timeout != -1 && !ReadWait (pbl, sizeof(hdr), timeout))
	    goto fail;

	bytes_read = ReadWithTimeout (pbl, hdr, sizeof(hdr), timeout);

	if(bytes_read == 0 && timeout == 0) 
            goto fail;

	if(bytes_read != sizeof(hdr)) {
	    //MessageBox(NULL, TEXT("Serial Receive Error!"), TEXT("Error"), MB_OK);
	    goto fail;
	}

/*
	TCHAR buf[100];
	wsprintf(buf, TEXT("Num bytes: %d Chars: %c%c%c", bytes_read, ((char *)hdr)[0],
		((char *)hdr)[1], ((char *)hdr)[2]);
	MessageBox(NULL, buf, TEXT("OK", MB_OK);
*/

	switch (cmd = PEBBLES_CMD(hdr)) {
	    case CMD_PEBBLES_ACK_CHANGE_PLUGIN:
                // PebblesPC acknowledged our CMD_PEBBLES_CHANGE_PLUGIN,
                // so we're safely connected to a plugin
#ifdef CONNECTED_SOUND
			if (!pbl->everConnected)
				MessageBeep (0);			
#endif
			pbl->connected = TRUE;
			pbl->everConnected = TRUE;

			if(pbl->connType == NETWORK) {
				// change the socket sending mode so wince doesn't clump
				// all the small packets together.
				int nodelay = true;
				if(setsockopt (HTOS(pbl->hPort), IPPROTO_TCP, TCP_NODELAY, (char *)&nodelay, sizeof(int)) == SOCKET_ERROR) {
					//wsprintf (szError, TEXT("setsockopt Failed. Error: %d"), WSAGetLastError ()); 
					MessageBox (NULL, TEXT("Error with setsockopt"), TEXT("Error"), MB_OK); 
					closesocket(HTOS(pbl->hPort));
					WSACleanup();
					return NULL;
				}
				
			}
			
            if (pbl->connectionCallback)
                pbl->connectionCallback (pbl);
            break;
		case CMD_PEBBLES_NO_ACK_CHANGE_PLUGIN:
			// The plugin was not found
			if(!pbl->pluginConnectError) {
				TCHAR buf[120];
				TCHAR plugBuf[30];

				pbl->pluginConnectError = TRUE;
#ifdef _WIN32_WCE
				MultiByteToWideChar(GetACP(), MB_PRECOMPOSED, pbl->plugin, strlen(pbl->plugin), plugBuf, 30);
#else
				strcpy (plugBuf, pbl->plugin);
#endif
				// For some reason the previous function doesn't always null terminate
				plugBuf[strlen(pbl->plugin)] = '\0';
				wsprintf(buf, TEXT("%s.dll was not found on the host computer.  Please place this file in the PebblesPC directory"), plugBuf);
				MessageBox(NULL, buf, TEXT("Plugin not Found"), MB_OK | MB_ICONERROR);
			}
		break;
	    case CMD_PEBBLES_STARTUP:
		// PebblesPC just started up; send it the
		// CMD_PEBBLES_CHANGE_PLUGIN message it probably missed
		PebblesChangePlugin (pbl, pbl->plugin, pbl->flags);
		break;
	    
	    case CMD_PEBBLES_KEEPALIVE:
		return CMD_PEBBLES_KEEPALIVE;
		break;

	    default:
		if (cmd >= CMD_PEBBLES_BASE && cmd <= CMD_PEBBLES_MAX) {
			// user-defined message -- return it
			*pLen = PEBBLES_LEN (hdr);
			if(*pLen == PEBBLES_LONG_MESSAGE_SIZE) {
				char exHdr[PEBBLES_LONG_HEADER_SIZE];
				bytes_read = ReadWithTimeout (pbl, exHdr+3, (PEBBLES_LONG_HEADER_SIZE-PEBBLES_HEADER_SIZE), timeout);
				*pLen = PEBBLES_LONG_LEN(exHdr);
			}
			return cmd;
		}
		break;
	}

fail:	// no message waiting
        if (!pbl->connected 
            && pbl->plugin != NULL
            && GetTickCount () >= pbl->repeatDeadline) {
            // Still not connected -- repeat the CHANGE_PLUGIN message
#ifdef PEBBLES_DEBUG
			char debugString[50];
			sprintf(debugString, "%d: Connection Failing...\n", GetTickCount());
			DebugWriteFile(debugString, strlen(debugString));
#endif
			if(pbl->connType == NETWORK) {
				//if(MessageBox(NULL, TEXT("Communication to the PC has been lost.  Reconnect?"), TEXT("Error", MB_OKCANCEL | MB_ICONEXCLAMATION) == IDOK) {
					PebblesClose(pbl);
					pbl->everConnected = false;
					PebblesOpen(pbl, pbl->plugin, NETWORK, pbl->ipAddr, pbl->flags);
				//}
			}
			else {
				if(!PebblesChangePlugin (pbl, pbl->plugin, pbl->flags)) {
#ifdef PEBBLES_DEBUG
					char dbgString[50];
					strcpy(dbgString, "Can't write to serial!\n");
					DebugWriteFile(dbgString, strlen(dbgString));
#endif				
				}
			}			
		}

	return 0;
}

/***********************************************************************
 *
 * FUNCTION:    PebblesReceiveData
 *
 * DESCRIPTION: Receives a message's data block.  May be called
 *              several times to receive an entire message, if
 *              desired.
 *
 * PARAMETERS:  pbl -- connection state
 *              data -- pointer to buffer for received data block.
 *                      May be NULL, in which case the bytes are
 *                      read and discarded.
 *              len -- length of data block, may be 0
 *				timeout -- interbyte timeout in ticks, i.e. 1/100s of 
 * 				           a second. (0 means poll, -1 means forever)
 *
 * RETURNED:    True if data received successfully; false if error.
 *              When false is returned, some of the data may have 
 *              been read.
 *
 ***********************************************************************/

BOOL PebblesReceiveData (PebblesPtr pbl, void *data, unsigned int len, int timeout, void (*callback) (int percentComplete))
{
	return ReadWithTimeout (pbl, (char*)data, len, timeout, callback) == len;
}


/***********************************************************************
 *
 * FUNCTION:    PebblesReceiveFlush
 *
 * DESCRIPTION: Throws away any characters waiting to be read.
 *              Might be used to discard a garbled message.
 *
 * PARAMETERS:  pbl -- connection state
 *
 * RETURNED:    nothing
 *
 ***********************************************************************/

void PebblesReceiveFlush (PebblesPtr pbl)
{
	PurgeComm(pbl->hPort, PURGE_RXCLEAR);
}

void PebblesSaveConnectionOptions(PebblesPtr pbl) {

	DWORD err, createFlag;
	HKEY hNew = NULL;

	// Save all options in registry
	err = RegCreateKeyEx(HKEY_CURRENT_USER, 
			TEXT("Software\\Pebbles"), 
			0,
			NULL,
			0, 0, NULL,
			&hNew, &createFlag);
	if(err == ERROR_SUCCESS) {
	 RegSetValueEx(hNew, TEXT("ConnType"), 0, REG_BINARY, (unsigned char *) &pbl->connType, sizeof(PebblesConnectionType));
	 RegSetValueEx(hNew, TEXT("Hostname"), 0, REG_SZ, (unsigned char *) &pbl->ipAddr, 16);

	 RegCloseKey(hNew);
	} 	

}


void PebblesLoadConnectionOptions(PebblesPtr pbl) {
    DWORD err, queryType, querySize;
    HKEY hNew = NULL;
	unsigned char connType = SERIAL;
	unsigned char szHostname[16];
	szHostname[0] = 0;

    err = RegOpenKeyEx(HKEY_CURRENT_USER, 
			TEXT("Software\\Pebbles"), 
			0,
			KEY_READ,
			&hNew);
    if(err == ERROR_SUCCESS) {
     querySize = 1;
     RegQueryValueEx(hNew, TEXT("ConnType"), 0, &queryType, &connType, &querySize);

	 querySize = 16;
	 RegQueryValueEx(hNew, TEXT("Hostname"), 0, &queryType, szHostname, &querySize);

     RegCloseKey(hNew);
    }

	pbl->connType = (PebblesConnectionType) connType;
	for(int i=0; i<16; i++)
		pbl->ipAddr[i] = szHostname[i];
}

#ifdef PEBBLESCE_EXPORTS
BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
    switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH:
			break;
    }
    return TRUE;
}
#endif
