//
// Pebbles library for WinCE.
//
// This file defines a set of functions that implement the app-side
// of the Pebbles protocol, for WinCE apps.
//

#ifndef _PEBBLES_CE_H_
#define _PEBBLES_CE_H_

#include <windows.h>


// The following ifdef block is the standard way of creating macros which make exporting 
// from a DLL simpler.
//     The PebblesCE DLL should define PEBBLESCE_EXPORTS.
//     Programs that import the DLL should define PEBBLESCE_IMPORTS.
//     Programs that statically link PebblesCE.cpp shouldn't define either symbol.
#ifdef PEBBLESCE_EXPORTS
#define PEBBLESCE_API __declspec(dllexport)
#else
#ifdef PEBBLESCE_IMPORTS
#define PEBBLESCE_API __declspec(dllimport)
#else
#define PEBBLESCE_API 
#endif
#endif

//
// Connection state.
//

// Device types.
typedef enum {
    SERIAL   = 0,
    INFRARED = 1,
	NETWORK   = 2
} PebblesConnectionType;


// Description of a Pebbles connection
typedef struct _PebblesConnection {

	char *	  plugin;		// Name of plugin to use on desktop (null-terminated)
	int flags;				// Flags to use with CHANGE_PLUGIN message

        PebblesConnectionType connType; // Network, Serial or infrared

        HANDLE	  hPort;		// Handle of open port library
	
	BOOL	  connected;            // True iff connected to an active plugin on
                                        // the desktop (i.e., a CMD_ACK has been
                                        // received from the desktop)
	BOOL	  everConnected;       // True iff connected has been true at least once
									// since last call to PebblesOpen()
									
	char *ipAddr;				// IP Address of remote computer

	BOOL	pluginConnectError;   	// True if a connection to the PC was made,
									// but no plugin .DLL was found

	HANDLE    hWatchdogThread;	// Thread watching for power-cycle

        void (*connectionCallback) (struct _PebblesConnection *);
                                        // Callback when connection is made

        DWORD     repeatDeadline;       // Tick count (see WinCE API function GetTickCount)
                                        // at which to repeat the CHANGE_PLUGIN message
} PebblesConnection;

typedef PebblesConnection *PebblesPtr;



//
// API functions.
//

PEBBLESCE_API BOOL PebblesOpen (PebblesPtr pbl, char *plugin, PebblesConnectionType connType = SERIAL, char *networkAddr=NULL, int flags = 0);
PEBBLESCE_API void PebblesClose (PebblesPtr pbl);

PEBBLESCE_API BOOL PebblesChangePlugin (PebblesPtr pbl, char *plugin, int flags = 0);
PEBBLESCE_API BOOL PebblesHandshake (PebblesPtr pbl);

PEBBLESCE_API BOOL PebblesSend (PebblesPtr pbl, char cmd, void *data, unsigned len);
PEBBLESCE_API BOOL PebblesSendHeader (PebblesPtr pbl, char cmd, unsigned len);
PEBBLESCE_API BOOL PebblesSendData (PebblesPtr pbl, void *data, unsigned len);

PEBBLESCE_API void PebblesSetConnectionCallback (PebblesPtr pbl, void (*callback) (PebblesPtr pbl));

PEBBLESCE_API char PebblesReceiveHeader (PebblesPtr pbl, unsigned int *pLen, int timeout);
PEBBLESCE_API BOOL PebblesReceiveData (PebblesPtr pbl, void *data, unsigned int len, int timeout, void (*callback) (int percentComplete) = NULL);

PEBBLESCE_API void PebblesReceiveFlush (PebblesPtr pbl);

PEBBLESCE_API void PebblesSetUserName (const char *name);

#endif
